/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.invoice.dao;

import gz.cyberbrain.cyberhotel.model.invoice.to.InvoiceTo;
import gz.cyberbrain.cyberhotel.model.spring.dao.AbstractSpringDao;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

public class InvoiceSpringDao extends AbstractSpringDao implements InvoiceDao {

	public InvoiceSpringDao() {
		super(InvoiceTo.class);
	}	

	public InvoiceTo create(InvoiceTo invoiceTo) throws InternalErrorException {		

		return (InvoiceTo) super.save(invoiceTo);

	}

	public InvoiceTo find(Long invoiceNumber, String invoiceSerial) throws InstanceNotFoundException, InternalErrorException {

		String queryName = "anInvoice";
		Serializable []attributes = {invoiceNumber, invoiceSerial}; 
		Collection invoiceTo = new ArrayList();
		invoiceTo = super.findByAttributes(queryName,attributes);
		Exception e = new Exception("Invoice not unique!");
		if (invoiceTo.size() > 1) throw new InternalErrorException(e);
		if (invoiceTo.size() == 0) throw new InstanceNotFoundException(attributes,InvoiceTo.class.getName());
		Iterator iter = invoiceTo.iterator();
		return (InvoiceTo) iter.next();

	}

	public void update(InvoiceTo invoiceTo) throws InternalErrorException {		

		super.update(invoiceTo);

	}

	public void remove(InvoiceTo invoiceTo) throws InternalErrorException {

		super.delete(invoiceTo);

	}

	public Collection findAll() throws InternalErrorException {

		String queryName = "allInvoices";
		return super.findByQuery(queryName);

	}

	public Collection findInvoiceOwes(Long invoiceNumber, String invoiceSerial) throws InternalErrorException {

		String queryName = "invoiceOwes";
		Serializable []attributes = {invoiceNumber, invoiceSerial};
		return super.findByAttributes(queryName, attributes);

	}

	public Collection findOutstanding() throws InternalErrorException {
		String queryName = "outstandingInvoices";
		return super.findByQuery(queryName);
	}

	public Collection findIssued() throws InternalErrorException {

		String queryName = "issuedInvoices";
		return super.findByQuery(queryName);
	}

	public boolean payedInvoice(Long invoiceNumber, String invoiceSerial) throws InternalErrorException {
		String queryName = "payedInvoice";
		Serializable []attributes = {invoiceNumber, invoiceSerial};
		Collection payments = super.findByAttributes(queryName, attributes);
		return payments.size() != 0;
	}

}
