/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions;

import java.util.Collection;
import java.util.Iterator;

import gz.cyberbrain.cyberhotel.model.client.dao.ClientDao;
import gz.cyberbrain.cyberhotel.model.invoice.dao.InvoiceDao;
import gz.cyberbrain.cyberhotel.model.invoice.to.InvoiceTo;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.UnaceptableOweException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.UnaceptablePayerException;
import gz.cyberbrain.cyberhotel.model.owe.dao.OweDao;
import gz.cyberbrain.cyberhotel.model.owe.to.OweTo;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

public class InsertAnInvoiceAction {

	private InvoiceDao invoiceDao;
	private OweDao oweDao;
	private ClientDao agencyDao;
	private ClientDao companyDao;
	private ClientDao particularDao;
	private InvoiceTo invoiceTo;
	private Collection owes;
	
	public InsertAnInvoiceAction(InvoiceDao invoiceDao, OweDao oweDao, ClientDao agencyDao, ClientDao companyDao, ClientDao particularDao, InvoiceTo invoiceTo, Collection owes) {
		
		this.invoiceDao = invoiceDao;
		this.oweDao = oweDao;
		this.agencyDao = agencyDao;
		this.companyDao = companyDao;
		this.particularDao = particularDao;
		this.invoiceTo = invoiceTo;
		this.owes = owes;
		
	}
	
	public InvoiceTo execute() throws UnaceptablePayerException, UnaceptableOweException, InstanceNotFoundException, InternalErrorException {
		
		//Payer must exist
		
		Collection existentAgencies = agencyDao.findAllIds();
		Collection existentCompanies = companyDao.findAllIds();
		Collection existentParticulars = particularDao.findAllIds();
		
		Long currentPayerId = invoiceTo.getPayer();
		
		if (currentPayerId == null) throw new UnaceptablePayerException(currentPayerId);
		if(!existentParticulars.contains(currentPayerId)
				&& !existentAgencies.contains(currentPayerId)
				&& !existentCompanies.contains(currentPayerId)) throw new UnaceptablePayerException(currentPayerId);			
		
		//Insert the invoice	
		invoiceDao.create(invoiceTo);
		
		//Associate owes to invoice
		Iterator iter = owes.iterator();
		OweTo owe;
		
		for (int i=0; iter.hasNext();i++) {
			
			owe = oweDao.find((Long) iter.next());

			//Owes can't be billed
			if (owe.getStatus().equals("Facturado")) throw new UnaceptableOweException(owe.getOweNumber());
			
			owe.setInvoiceNumber(invoiceTo.getInvoiceNumber());
			owe.setInvoiceSerial(invoiceTo.getInvoiceSerial());
			owe.setStatus("Facturado");
			oweDao.update(owe);
			
		}
		
		return invoiceTo;
		
	}
	
}
