/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions;

import gz.cyberbrain.cyberhotel.model.cash.dao.CashDao;
import gz.cyberbrain.cyberhotel.model.cash.to.CashTo;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.ClosedPaymentException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.UnaceptablePaymentException;
import gz.cyberbrain.cyberhotel.model.payment.dao.PaymentDao;
import gz.cyberbrain.cyberhotel.model.payment.to.PaymentTo;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.Iterator;

public class UpdateACashAction {

	private CashDao cashDao;
	private PaymentDao paymentDao;
	private CashTo cashTo;
	private Collection payments;	
	private boolean closement;

	public UpdateACashAction(CashDao cashDao, PaymentDao paymentDao, CashTo cashTo, Collection payments, boolean closement) {
		this.cashDao = cashDao;
		this.paymentDao = paymentDao;
		this.cashTo = cashTo;
		this.payments = payments;
		this.closement = closement;
	}

	public void execute() throws ClosedCashException, UnaceptablePaymentException, ClosedPaymentException, InstanceNotFoundException, InternalErrorException {

		//Cash can't be opened.
		
		if (!closement && cashTo.getStatus().equals("Pechada")) throw new ClosedCashException(cashTo.getCashNumber());
		
		Iterator iter = payments.iterator();
		Long reference;
		PaymentTo paymentTo;

		long accountsNumber = new Long(cashTo.getAccountsNumber()).longValue();
		long cardsNumber =  new Long(cashTo.getCardsNumber()).longValue();
		long vouchersNumber =  new Long(cashTo.getVouchersNumber()).longValue();
		double accountsTotal =  cashTo.getAccountsTotal().doubleValue();
		double cardsTotal =  cashTo.getCardsTotal().doubleValue();
		double vouchersTotal =  cashTo.getVouchersTotal().doubleValue();
		double cashTotal =  cashTo.getCashTotal().doubleValue();

		for (int i=0; iter.hasNext(); i++) {

			reference = (Long) iter.next();

			//Payment must exist
			try {
				paymentTo = paymentDao.find(reference);
			} catch (InstanceNotFoundException e) {
				throw new UnaceptablePaymentException(reference);
			}

			//Payment can't be closed
			if (paymentTo.getStatus().equals("Pechado")) throw new ClosedPaymentException(reference);

			//Update cash values

			if (paymentTo.getPayMode().equals("Contado")) {
				accountsNumber += 1;
				accountsTotal += paymentTo.getAmount().doubleValue();
			}
			if (paymentTo.getPayMode().equals("Tarxeta")) {
				cardsNumber  += 1;
				cardsTotal += paymentTo.getAmount().doubleValue();
			}
			if (paymentTo.getPayMode().equals("Bono")) {
				vouchersNumber += 1;
				vouchersTotal += paymentTo.getAmount().doubleValue();
			}

			cashTotal = accountsTotal + cardsTotal + vouchersTotal;

			//Set payments status to closed

			paymentTo.setStatus("Pechado");
			paymentTo.setCash(cashTo.getCashNumber());
			paymentDao.update(paymentTo);

		}

		cashTo.setPaymentsNumber(payments.size());
		
		cashTo.setAccountsNumber(accountsNumber);
		cashTo.setCardsNumber(cardsNumber);
		cashTo.setVouchersNumber(vouchersNumber);
		cashTo.setAccountsTotal(new BigDecimal(accountsTotal));
		cashTo.setCardsTotal(new BigDecimal(cardsTotal));
		cashTo.setVouchersTotal(new BigDecimal(vouchersTotal));
		
		cashTo.setCashTotal(new BigDecimal(cashTotal));
		
		cashDao.update(cashTo);

	}

}
