/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.invoicingfacade.spring.actions;

import gz.cyberbrain.cyberhotel.model.invoice.dao.InvoiceDao;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.ClosedPaymentException;
import gz.cyberbrain.cyberhotel.model.invoicingfacade.exceptions.IssuedInvoiceException;
import gz.cyberbrain.cyberhotel.model.payment.dao.PaymentDao;
import gz.cyberbrain.cyberhotel.model.payment.to.PaymentTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableInvoiceException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;

public class UpdateAPaymentAction {

	private InvoiceDao invoiceDao;
	private PaymentDao paymentDao;
	private PaymentTo paymentTo;

	public UpdateAPaymentAction(InvoiceDao invoiceDao, PaymentDao paymentDao, PaymentTo paymentTo) {		
		this.invoiceDao = invoiceDao;
		this.paymentDao = paymentDao;
		this.paymentTo = paymentTo;
	}

	public void execute() throws ClosedPaymentException, UnaceptableInvoiceException, IssuedInvoiceException, InstanceNotFoundException, InternalErrorException {

		//Closed payments can't be editables
		
		if (paymentTo.getStatus().equals("Pechado")) throw new ClosedPaymentException(paymentTo.getReference());
		
		//Invoice must exist
		
		try {
			invoiceDao.find(paymentTo.getInvoiceNumber(), paymentTo.getInvoiceSerial());	
		} catch (InstanceNotFoundException e) {
			throw new UnaceptableInvoiceException(paymentTo.getInvoiceNumber(), paymentTo.getInvoiceSerial());
		} catch (InternalErrorException e) {
			throw e;
		}		

		paymentDao.update(paymentTo);

	}

}
