/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Collection;
import java.util.Iterator;

import gz.cyberbrain.cyberhotel.model.concept.dao.ConceptDao;
import gz.cyberbrain.cyberhotel.model.concept.to.ConceptTo;
import gz.cyberbrain.cyberhotel.model.day.dao.DayDao;
import gz.cyberbrain.cyberhotel.model.owe.dao.OweDao;
import gz.cyberbrain.cyberhotel.model.owe.to.OweTo;
import gz.cyberbrain.cyberhotel.model.price.dao.PriceDao;
import gz.cyberbrain.cyberhotel.model.price.to.PriceTo;
import gz.cyberbrain.cyberhotel.model.request.to.RequestTo;
import gz.cyberbrain.cyberhotel.model.reservation.dao.ReservationDao;
import gz.cyberbrain.cyberhotel.model.reservation.to.ReservationTo;
import gz.cyberbrain.cyberhotel.model.reservationfacade.delegate.ReservationFacadeDelegate;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.CancelledReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.DifferentDayException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.FinishedReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.PendingReservationException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.PriceNotFoundException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.RoomNotBusyException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableConceptException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableInvoiceException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableParticularException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.UnaceptableRoomException;
import gz.cyberbrain.cyberhotel.model.room.dao.RoomDao;
import gz.cyberbrain.cyberhotel.model.room.to.RoomTo;
import gz.cyberbrain.cyberhotel.model.spring.facade.SpringFacadeFactory;
import gz.cyberbrain.cyberhotel.model.util.config.ConfigurationHandler;
import gz.cyberbrain.cyberhotel.model.util.dates.DateOperations;
import gz.cyberbrain.cyberhotel.model.util.dates.Interval;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InstanceNotFoundException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InvalidIntervalException;
import gz.cyberbrain.cyberhotel.model.util.money.MoneyOperations;

public class CheckOutAction {

	private ReservationTo reservationTo;
	private Calendar trueOutgoingDate;
	private ReservationDao reservationDao;
	private RoomDao roomDao;
	private ConceptDao conceptDao;	
	private DayDao dayDao;
	private PriceDao priceDao;
	private OweDao oweDao;

	public CheckOutAction (ReservationTo reservationTo, Calendar trueOutgoingDate, ReservationDao reservationDao, 
			RoomDao roomDao, ConceptDao conceptDao, DayDao dayDao, PriceDao priceDao, OweDao oweDao) {
		this.reservationTo = reservationTo;
		this.trueOutgoingDate = trueOutgoingDate;
		this.reservationDao = reservationDao;
		this.roomDao = roomDao;
		this.conceptDao = conceptDao;
		this.dayDao = dayDao;
		this.priceDao = priceDao;
		this.oweDao = oweDao;

	}

	public void execute() 
	throws  DifferentDayException, PendingReservationException, 
	CancelledReservationException, FinishedReservationException, PriceNotFoundException,    
	InvalidIntervalException, InstanceNotFoundException, InternalErrorException, 
	UnaceptableRoomException, UnaceptableConceptException, UnaceptableParticularException, 
	UnaceptableInvoiceException, RoomNotBusyException {		

		if (reservationTo.getReservationStatus().equals("Pendente"))
			throw new PendingReservationException(reservationTo);

		if (reservationTo.getReservationStatus().equals("Anulada"))
			throw new CancelledReservationException(reservationTo);

		if (reservationTo.getReservationStatus().equals("Finalizada")) 
			throw new FinishedReservationException(reservationTo);

		Collection requests = reservationDao.findRequests(reservationTo.getReservationId());

		Iterator iter = requests.iterator();

		RoomTo updatedRoom;

		for (int i=0; iter.hasNext();i++) {
			updatedRoom = roomDao.find(((RequestTo) iter.next()).getRoom());
			updatedRoom.setOccupancyStatus("Libre");
			updatedRoom.setCleaningStatus("Suxa");
			roomDao.update(updatedRoom);
		}

		/* Owes management*/

		Collection reservationOwes = reservationDao.findReservationOwes(reservationTo.getReservationId());
		System.out.println("CHECKOUTACTION: " + reservationOwes.size());
		Iterator owes = reservationOwes.iterator();
		OweTo currentOwe;

		ConceptTo conceptTo = conceptDao.find(ConfigurationHandler.STAY_CONCEPT);
		BigDecimal vat = conceptTo.getVat();						

		String board = null;
		String priceType = null;
		Long season = null;
		Long priceId;

		BigDecimal price;
		BigDecimal amount;
		BigDecimal discount = new BigDecimal(0); //No offers implemented
		String status = "Pendente";
		Long particular;
		String room;
		Long invoiceNumber = null;
		String invoiceSerial = null;
		String concept = conceptTo.getConceptCode();	

		if (reservationTo.getByHours()) {
			if ((reservationTo.getExpectedOutgoingDate()).get(Calendar.DAY_OF_YEAR) != (trueOutgoingDate).get(Calendar.DAY_OF_YEAR)) 
				throw new DifferentDayException(reservationTo);
			priceId = new Long(2);
		}
		else 
			priceId = new Long(1);

		//Owes elimination or creation.

		if (trueOutgoingDate.before(reservationTo.getExpectedOutgoingDate())) {
			//Owes elimination (Only stay owes)
			for (int i=0; owes.hasNext(); i++) {				
				currentOwe = oweDao.find(new Long((Short) owes.next())); 
				if ((currentOwe.getOweDate()).after(trueOutgoingDate) && (currentOwe.getConcept().equals(concept))) oweDao.remove(currentOwe);
			}
		}  

		if (trueOutgoingDate.after(reservationTo.getExpectedOutgoingDate())){
			Interval interval = new Interval(reservationTo.getExpectedOutgoingDate(),trueOutgoingDate);
			//Owes creation

			Calendar nextDate = (Calendar) (reservationTo.getExpectedOutgoingDate()).clone(); 

			RequestTo currentRequest;
			PriceTo currentPrice;

			for (int i=0; i<DateOperations.getIntervalDifference(interval); i++) {

				nextDate.add(Calendar.DATE, 1);

				try {
					season = (dayDao.find((Calendar) nextDate.clone())).getSeason();
				} catch (InstanceNotFoundException e) {
					throw new PriceNotFoundException(priceId, board, priceType, season);
				}

				iter = requests.iterator();

				for (int j=0; iter.hasNext();j++) {

					currentRequest = (RequestTo) iter.next(); 

					board = (currentRequest).getBoard();

					priceType = (currentRequest).getPriceType();

					try {				
						currentPrice = priceDao.find(priceId, board, priceType, season);
					} catch (InstanceNotFoundException e) {
						throw new PriceNotFoundException(priceId, board, priceType, season);
					}

					price = (currentPrice).getValue();
					amount = MoneyOperations.getPriceWithVat(price,vat);

					particular = currentRequest.getParticular();
					room = currentRequest.getRoom();

					ReservationFacadeDelegate reservationFacade = SpringFacadeFactory.getReservationFacadeDelegate(); 

					reservationFacade.oweAddition(new OweTo(null,(Calendar) nextDate.clone(),price,vat,amount,discount,status,particular, room,invoiceNumber,invoiceSerial,concept));

				}
			}

		}

		reservationTo.setReservationStatus("Finalizada");

		reservationDao.update(reservationTo);

	}

}
