/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.model.reservationfacade.spring.actions;

import gz.cyberbrain.cyberhotel.model.client.dao.ClientDao;
import gz.cyberbrain.cyberhotel.model.client.to.ParticularTo;

import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientPayerException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ClientWithInvoicesException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ParticularWithOwesException;
import gz.cyberbrain.cyberhotel.model.reservationfacade.exceptions.ParticularWithReservationsException;
import gz.cyberbrain.cyberhotel.model.util.exceptions.InternalErrorException;


public class RemoveAParticularAction {

	private ParticularTo particularTo;
	private ClientDao particularDao;
	
	public RemoveAParticularAction(ParticularTo particularTo, ClientDao particularDao) {
		this.particularTo = particularTo;
		this.particularDao = particularDao;
	}
	
	public void execute() 
		throws ParticularWithOwesException, ParticularWithReservationsException, 
		ClientPayerException, ClientWithInvoicesException, InternalErrorException {
		
		if (particularDao.existOwes(particularTo.getClientId()))
			throw new ParticularWithOwesException(particularTo);
		
		if (particularDao.existRequests(particularTo.getClientId()))
			throw new ParticularWithReservationsException(particularTo);
		
		if (particularDao.isPayer(particularTo.getClientId()))
			throw new ClientPayerException(particularTo);
		
		if (particularDao.existInvoices(particularTo.getClientId()))
			throw new ClientWithInvoicesException(particularTo);
		
		particularDao.remove(particularTo);
		
	}
	
}
