/************************************************************************************************/
/*                                                                                              */
/*      <cyberHotel: Gestion Hotelera GPL>                                                      */
/*      Copyright (C) <2008> <davide05@gmail.com>                                               */
/*                                                                                              */
/*      This program is free software: you can redistribute it and/or modify                    */
/*      it under the terms of the GNU General Public License as published by                    */
/*      the Free Software Foundation, either version 3 of the License, or                       */
/*      (at your option) any later version.                                                     */
/*                                                                                              */
/*      This program is distributed in the hope that it will be useful,                         */
/*      but WITHOUT ANY WARRANTY; without even the implied warranty of                          */
/*      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                                    */ 
/*      See the GNU General Public License for more details.                                    */
/*                                                                                              */
/*      You should have received a copy of the GNU General Public License                       */
/*      along with this program.  If not, see <http://www.gnu.org/licenses/>.                   */
/*                                                                                              */
/************************************************************************************************/

package gz.cyberbrain.cyberhotel.view.swing.edit;

import gz.cyberbrain.cyberhotel.controller.RoomTypeListAdapter;
import gz.cyberbrain.cyberhotel.controller.SaveAdapter;
import gz.cyberbrain.cyberhotel.controller.util.ErrorsHandler;
import gz.cyberbrain.cyberhotel.view.swing.FillableWindow;
import gz.cyberbrain.cyberhotel.view.swing.start.MainWindow;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.InfoPanel;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.IntegerDocument;
import gz.cyberbrain.cyberhotel.view.swing.util.gui.TextItem;
import gz.cyberbrain.cyberhotel.view.swing.util.locale.Locale;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.io.Serializable;
import java.util.HashMap;

import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel; 
import javax.swing.JLabel;
import javax.swing.ButtonGroup;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.border.LineBorder;
import javax.swing.text.Document;

public class RoomWindow extends FillableWindow {

	private static final long serialVersionUID = 1L;

	private static RoomWindow roomWindow = null;

	private SaveAdapter saveAdapter = null;

	private JInternalFrame owner = null;

	private Serializable id = null;

	private String dialogMessage = null;  //  @jve:decl-index=0:

	private String dialogTitle = null;

	private JPanel jContentPane = null;

	private JLabel codeLabel = null;

	private JTextField codeTextField = null;

	private JLabel numLabel = null;

	private JTextField numTextField = null;

	private JComboBox typeComboBox = null;

	private JLabel typeLabel = null;

	private JButton typeButton = null;

	private JRadioButton smokeRadioButton = null;

	private JRadioButton nonSmokeRadioButton = null;

	private JLabel paxLabel = null;

	private JTextField paxTextField = null;

	private JLabel floorLabel = null;

	private JTextField floorTextField = null;

	private JLabel notesLabel = null;

	private JTextArea notesTextArea = null;

	private ButtonGroup buttonGroup = null;

	private InfoPanel infoPanel = null;

	private JButton saveButton = null;

	private JButton cancelButton = null;

	private JScrollPane jScrollPane = null;

	/**
	 * @param owner
	 */
	private RoomWindow() {
		super("", false, true, false, true);
		initialize();
	}

	public static RoomWindow getInstance() {
		if (roomWindow == null) roomWindow = new RoomWindow();		
		return roomWindow;
	}

	public void setAdapter(SaveAdapter saveAdapter) {
		this.saveAdapter = saveAdapter;
	}

	public SaveAdapter getAdapter() {
		return saveAdapter;
	}

	public String getDialogMessage() {
		return dialogMessage;
	}

	public void setDialogMessage(String dialogMessage) {
		this.dialogMessage = dialogMessage;
	}

	public String getDialogTitle() {
		return dialogTitle;
	}

	public void setDialogTitle(String dialogTitle) {
		this.dialogTitle = dialogTitle;
	}

	public JInternalFrame getOwner() {
		return owner;
	}

	public void setOwner(JInternalFrame owner) {
		this.owner = owner;
	}

	public void setId(Serializable id) {
		this.id = id;
	}

	public Serializable getId() {
		return id;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {		
		this.setSize(468, 256);
		this.setLocation(387,0);
		this.setResizable(false);
		this.setContentPane(getJContentPane());		
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		this.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter() {

			public void internalFrameClosing(javax.swing.event.InternalFrameEvent evt) {

				(RoomWindow.getInstance()).clean();
				(RoomWindow.getInstance()).dispose();

			}

			public void internalFrameActivated(javax.swing.event.InternalFrameEvent evt) {

				jContentPane = null;
				typeComboBox = null;
				(RoomWindow.getInstance()).setContentPane(getJContentPane());
				roomWindow = null;
			}

		});
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {	 	         
			buttonGroup = new ButtonGroup();
			buttonGroup.add(getNonSmokeRadioButton());
			buttonGroup.add(getSmokeRadioButton());
			notesLabel = new JLabel();
			notesLabel.setBounds(new Rectangle(17, 134, 47, 16));
			notesLabel.setText(Locale.get("RoomWindow.notesLabel"));
			floorLabel = new JLabel();
			floorLabel.setBounds(new Rectangle(342, 103, 36, 16));
			floorLabel.setText(Locale.get("RoomWindow.floorLabel"));
			paxLabel = new JLabel();
			paxLabel.setBounds(new Rectangle(261, 103, 33, 16));
			paxLabel.setText(Locale.get("RoomWindow.paxLabel"));
			typeLabel = new JLabel();
			typeLabel.setBounds(new Rectangle(17, 73, 44, 16));
			typeLabel.setText(Locale.get("RoomWindow.typeLabel"));
			numLabel = new JLabel();
			numLabel.setBounds(new Rectangle(138, 103, 52, 16));
			numLabel.setText(Locale.get("RoomWindow.numLabel"));
			codeLabel = new JLabel();
			codeLabel.setBounds(new Rectangle(17, 103, 47, 16));
			codeLabel.setText(Locale.get("RoomWindow.codeLabel"));
			jContentPane = new JPanel();
			jContentPane.setLayout(null);
			jContentPane.add(codeLabel, null);
			jContentPane.add(getCodeTextField(), null);
			jContentPane.add(numLabel, null);
			jContentPane.add(getNumTextField(), null);
			jContentPane.add(getTypeComboBox(), null);
			jContentPane.add(typeLabel, null);
			jContentPane.add(getTypeButton(), null);
			jContentPane.add(getSmokeRadioButton(), null);
			jContentPane.add(getNonSmokeRadioButton(), null);
			jContentPane.add(paxLabel, null);
			jContentPane.add(getPaxTextField(), null);
			jContentPane.add(floorLabel, null);
			jContentPane.add(getFloorTextField(), null);
			jContentPane.add(notesLabel, null);
			jContentPane.add(getInfoPanel(),null);
			jContentPane.add(getSaveButton(), null);
			jContentPane.add(getCancelButton(), null);
			jContentPane.add(getJScrollPane(), null);
		}
		return jContentPane;
	}

	/**
	 * This method initializes codeTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getCodeTextField() {
		if (codeTextField == null) {
			codeTextField = new JTextField();
			codeTextField.setBounds(new Rectangle(69, 103, 63, 19));
		}
		return codeTextField;
	}

	/**
	 * This method initializes numTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getNumTextField() {
		if (numTextField == null) {
			numTextField = new JTextField();
			numTextField.setBounds(new Rectangle(194, 103, 56, 19));
		}
		return numTextField;
	}

	/**
	 * This method initializes typeComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */
	private JComboBox getTypeComboBox() {
		if (typeComboBox == null) {
			typeComboBox = new JComboBox((new RoomTypeListAdapter()).load());
			typeComboBox.setBounds(new Rectangle(69, 73, 75, 18));			
		}
		return typeComboBox;
	}		

	/**
	 * This method initializes typeButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getTypeButton() {
		if (typeButton == null) {
			typeButton = new JButton();
			typeButton.setText(Locale.get("RoomWindow.typeButton"));
			typeButton.setBounds(new Rectangle(154, 73, 96, 18));
			typeButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					InfoPanel ipanel = new InfoPanel();
					RoomTypeWindow roomTypeWindow = RoomTypeWindow.getInstance();
					ipanel.setBounds(0,0,roomTypeWindow.getWidth(),50);
					ipanel.setInfoText(Locale.get("RoomTypeWindow.infoText"));
					roomTypeWindow.setInfoPanel(ipanel);
					roomTypeWindow.setTitle(Locale.get("RoomTypeWindow.title"));
					roomTypeWindow.setDialogMessage(Locale.get("JOptionPane.RoomTypeWindow.saveButton.message"));
					roomTypeWindow.setDialogTitle(Locale.get("JOptionPane.RoomTypeWindow.saveButton.title"));
					roomTypeWindow.setAdapter((new RoomTypeListAdapter()).getCreator());					
					(MainWindow.getInstance()).showInDesktopPane(roomTypeWindow);

				}
			});
		}
		return typeButton;
	}

	/**
	 * This method initializes smokeRadioButton	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */
	private JRadioButton getSmokeRadioButton() {
		if (smokeRadioButton == null) {
			smokeRadioButton = new JRadioButton();
			smokeRadioButton.setText(Locale.get("RoomWindow.smokeRadioButton"));			 
			smokeRadioButton.setBounds(new Rectangle(361, 73, 85, 21));
		}
		return smokeRadioButton;
	}

	/**
	 * This method initializes nonSmokeRadioButton	
	 * 	
	 * @return javax.swing.JRadioButton	
	 */
	private JRadioButton getNonSmokeRadioButton() {
		if (nonSmokeRadioButton == null) {
			nonSmokeRadioButton = new JRadioButton();
			nonSmokeRadioButton.setText(Locale.get("RoomWindow.nonSmokeRadioButton"));
			nonSmokeRadioButton.setBounds(new Rectangle(260, 73, 94, 21));
			nonSmokeRadioButton.doClick();
		}
		return nonSmokeRadioButton;
	}

	/**
	 * This method initializes paxTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getPaxTextField() {
		if (paxTextField == null) {
			Document integerDocument = new IntegerDocument(3);
			paxTextField = new JTextField(integerDocument,"",3);
			paxTextField.setBounds(new Rectangle(299, 103, 30, 19));
			paxTextField.setText("0");
		}
		return paxTextField;
	}

	/**
	 * This method initializes floorTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getFloorTextField() {
		if (floorTextField == null) {
			Document integerDocument = new IntegerDocument(3);
			floorTextField = new JTextField(integerDocument,"",3);
			floorTextField.setBounds(new Rectangle(390, 103, 30, 19));
			floorTextField.setText("0");
		}
		return floorTextField;
	}

	/**
	 * This method initializes notesTextArea	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getNotesTextArea() {
		if (notesTextArea == null) {
			notesTextArea = new JTextArea();
		}
		return notesTextArea;
	}

	private InfoPanel getInfoPanel() {
		if (infoPanel == null) {
			infoPanel = new InfoPanel();
			infoPanel.setBounds(0,0,this.getWidth(),50);
		}
		return infoPanel;
	}

	public void setInfoPanel(InfoPanel ipanel) {		  

		jContentPane.remove(infoPanel);
		infoPanel = ipanel;
		jContentPane.add(infoPanel);

	}

	/**
	 * This method initializes saveButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getSaveButton() {
		if (saveButton == null) {
			saveButton = new JButton();
			saveButton.setText(Locale.get("RoomWindow.saveButton"));
			saveButton.setBounds(new Rectangle(240, 197, 90, 17));
			saveButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {					

					HashMap<String, Object> attrs = new HashMap<String, Object>();
					attrs.put("code",getCodeTextField().getText());
					attrs.put("number", getNumTextField().getText());

					try {

						attrs.put("floor", new Long(getFloorTextField().getText()));

					} catch (NumberFormatException e1) {
						attrs.put("floor", new Long("0"));
					}

					if (getTypeComboBox().getItemCount() != 0)
						attrs.put("type", (String) ((TextItem) getTypeComboBox().getSelectedItem()).getId());
					else 
						attrs.put("type", "");

					attrs.put("smokers", new Boolean(getSmokeRadioButton().isSelected()));

					try {

						attrs.put("maxPax",new Long(getPaxTextField().getText()));

					} catch (NumberFormatException e1) {
						attrs.put("maxPax",new Long("0"));
					}

					attrs.put("notes", getNotesTextArea().getText());

					String message = getDialogMessage();
					String title = getDialogTitle();

					getAdapter().save(attrs);

					if (!inputErrors()) {

						Object[] options = {Locale.get("JOptionPane.Ok")};
						int n = JOptionPane.showOptionDialog(RoomWindow.getInstance(),
								message,
								title,
								JOptionPane.OK_OPTION,
								JOptionPane.INFORMATION_MESSAGE,
								null,
								options,
								options[0]);

						if (n == JOptionPane.OK_OPTION) {
							ListWindow listWindow = (ListWindow) getOwner();						
							listWindow.loadList();
							clean();
							dispose();	                    	

						}					

					}

				}

			});

		}
		return saveButton;
	}

	/**
	 * This method initializes cancelButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getCancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton();
			cancelButton.setText(Locale.get("RoomWindow.cancelButton"));
			cancelButton.setBounds(new Rectangle(349, 197, 90, 17));
			cancelButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {

					(RoomWindow.getInstance()).clean();
					(RoomWindow.getInstance()).dispose();
					
				}
			});
		}
		return cancelButton;
	}

	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBounds(new Rectangle(69, 134, 347, 56));
			jScrollPane.setViewportView(getNotesTextArea());
		}
		return jScrollPane;
	}

	public void clean() {
		
		getInfoPanel().setInfoText(getInfoPanel().getInfoText());
		if (getTypeComboBox().getItemCount() != 0) getTypeComboBox().setSelectedIndex(0);
		getCodeTextField().setEditable(true);
		getCodeTextField().setText("");
		getNumTextField().setText("");
		getFloorTextField().setText("");
		getNonSmokeRadioButton().setSelected(true);
		getPaxTextField().setText("");
		getNotesTextArea().setText("");
		getCodeTextField().setBorder(UIManager.getBorder("TextField.border"));
		getNumTextField().setBorder(UIManager.getBorder("TextField.border"));		
		getTypeComboBox().setBorder(UIManager.getBorder("ComboBox.border"));

	}

	public void fill(HashMap fields) {

		getCodeTextField().setEditable(false);
		getCodeTextField().setText((String) fields.get("code"));
		getNumTextField().setText((String) fields.get("number"));
		getFloorTextField().setText(((Long) fields.get("floor")).toString());
		for (int i=0; i<getTypeComboBox().getItemCount(); i++) {
			TextItem item = (TextItem) getTypeComboBox().getItemAt(i); 
			if ((item.getId()).equals(((TextItem) fields.get("type")).getId())) {
				getTypeComboBox().setSelectedIndex(i);
			}
		}
		boolean smokers = ((Boolean) fields.get("smokers")).booleanValue();
		getSmokeRadioButton().setSelected(smokers);
		getNonSmokeRadioButton().setSelected(!smokers);
		getPaxTextField().setText(((Long) fields.get("maxPax")).toString());
		getNotesTextArea().setText((String) fields.get("notes"));		

	}

	private boolean inputErrors() {

		if (!ErrorsHandler.getErrors().isEmpty()) {

			if (ErrorsHandler.getErrorAt(0).getItem().equals("type")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("RoomWindow.errors.type")+ "</html>");

				getTypeComboBox().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;
			}


			if (ErrorsHandler.getErrorAt(0).getItem().equals("code")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("RoomWindow.errors.code")+ "</html>");

				getCodeTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;
			}

			if (ErrorsHandler.getErrorAt(0).getItem().equals("number")) {
				getInfoPanel().setErrorText("<html>" + Locale.get("InfoPanel.errors.title") + "<br>" 
						+ Locale.get("RoomWindow.errors.number")+ "</html>");

				getNumTextField().setBorder(new LineBorder(Color.red, 2));
				Toolkit.getDefaultToolkit().beep();
				return true;
			}

		}
		
		ErrorsHandler.clear();

		return false;

	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
